#ifdef _WIN64
#define NAME "Wargus (64 bit)"
#else
#define NAME "Wargus"
#endif

#define DESCRIPTION NAME
#define VERSION "2.4.1"
#define VIVERSION 2,4,1
#define HOMEPAGE "https://github.com/Wargus/wargus"
#define LICENSE "GPL v2"
#define COPYRIGHT "Copyright (c) 1998-2015 by The Stratagus Project"

1 ICON "wargus.ico"
1 VERSIONINFO
FILEVERSION	VIVERSION
PRODUCTVERSION	VIVERSION
FILEOS		4
FILETYPE	1

BEGIN
	BLOCK "StringFileInfo"
	BEGIN
		BLOCK "040904E4"
		BEGIN
			VALUE "FileDescription",        DESCRIPTION
			VALUE "FileVersion",            VERSION
			VALUE "InternalName",           NAME
			VALUE "LegalCopyright",         COPYRIGHT
			VALUE "License",                LICENSE
			VALUE "Homepage",               HOMEPAGE
			VALUE "OriginalFilename",       "wargus.exe"
			VALUE "ProductName",            NAME
			VALUE "ProductVersion",         VERSION
		END
	END

	BLOCK "VarFileInfo"
	BEGIN
		VALUE "Translation", 0x409, 1252
	END
END
